from collections import OrderedDict
from onegov.foundation import BaseTheme
from onegov.core.utils import module_path


class OnboardingTheme(BaseTheme):
    name = 'onegov.onboarding.foundation'

    # don't touch this number, it's incremented using bumpversion, so every
    # release will automatically trigger a rebuild of the theme
    version = '0.0.0'

    @property
    def post_imports(self):
        return [
            'onboarding'
        ]

    @property
    def default_options(self):
        return OrderedDict((
            ('primary-color', '#005BA1'),

            # base colors
            ('gray', '#e0e3e5'),
            ('black', '#0f0f0f'),
            ('blue', '#2575ed'),
            ('red', '#de2c3b'),
            ('yellow', '#ffc800'),
            ('orange', '#ffb100'),
            ('green', '#2c9f42'),
            ('white', '#fff'),
            ('gray-dark', '#b3b6b7'),
            ('black-dark', '#000'),
            ('blue-dark', '#1a52a5'),
            ('red-dark', '#b2232f'),
            ('yellow-dark', '#cca000'),
            ('orange-dark', '#d59200'),
            ('green-dark', '#237f35'),
            ('gray-light', '#f0f1f2'),
            ('black-light', '#363738'),
            ('blue-light', '#92baf6'),
            ('red-light', '#ef969d'),
            ('yellow-light', '#ffe480'),
            ('green-light', '#96cfa1'),
            # zurb overrides
            ('alert-color', '$red'),
            ('success-color', '$green'),
            ('warning-color', '$yellow'),
            ('info-color', '$blue'),

        ))

    @property
    def extra_search_paths(self):
        return [module_path('onegov.onboarding.theme', 'styles')]



import morepath
import re

from onegov.core import utils
from onegov.core.crypto import random_password
from onegov.core.templates import render_template
from onegov.onboarding import _
from onegov.onboarding.errors import AlreadyExistsError
from onegov.onboarding.forms import FinishForm, TownForm
from onegov.onboarding.layout import MailLayout
from onegov.onboarding.models.assistant import Assistant
from onegov.town.initial_content import create_new_organisation
from onegov.org.models import Organisation
from onegov.user import UserCollection

_valid_subdomain = re.compile(r'^[a-z0-9]+[a-z0-9-]+[a-z0-9]+$')


class TownAssistant(Assistant):
    """ An assistant guiding a user through onegov.town onboarding. """

    @Assistant.step(form=TownForm)
    def first_step(self, request, form):

        if form.submitted(request):
            request.browser_session['name'] = form.data['name']
            request.browser_session['user'] = form.data['user']
            request.browser_session['color'] = form.data['color'].get_hex()

            return morepath.redirect(request.link(self.for_next_step()))

        form.name.data = request.browser_session.get('name', form.name.data)
        form.user.data = request.browser_session.get('user', form.user.data)
        form.color.data = request.browser_session.get('color', form.color.data)

        return {
            'name': 'town-start',
            'title': _("Online Counter for Towns Demo"),
            'bullets': (
                _("Start using the online counter for your town immediately."),
                _("Setup takes less than one minute."),
                _("Free with no commitment.")
            ),
        }

    @Assistant.step(form=FinishForm)
    def last_step(self, request, form):

        for key in ('name', 'user', 'color'):
            if not request.browser_session.has(key):
                return morepath.redirect(request.link(self.for_prev_step()))

        name = request.browser_session['name']
        user = request.browser_session['user']
        color = request.browser_session['color']

        if form.submitted(request):
            try:
                product = self.add_town(name, user, color, request)
                error = None
            except AlreadyExistsError:
                product = None
                error = _(
                    "This town exists already and can't be created. Is it "
                    "your town but you did not create it? Please contact us."
                )
            else:
                self.app.notify_hipchat(
                    (
                        'A new OneGov Cloud instance was started by {}: '
                        '<a href="{}">{}</a>'
                    ).format(user, product['url'], name)
                )
            finally:
                del request.browser_session['name']
                del request.browser_session['user']
                del request.browser_session['color']

            if error:
                return {
                    'name': 'town-error',
                    'title': _("Online Counter for Towns Demo"),
                    'error': error,
                    'form': None
                }
            else:
                return {
                    'name': 'town-success',
                    'title': _("Online Counter for Towns Demo"),
                    'product': product,
                    'message': _("Success! Have a look at your new website!"),
                    'warning': _(
                        "Please write down your username and password "
                        "before you continue. "
                    ),
                    'form': None
                }

        return {
            'name': 'town-ready',
            'title': _("Online Counter for Towns Demo"),
            'message': _(
                "We are ready to launch! Click continue once you're ready."
            ),
            'preview': {
                'name': name,
                'user': user,
                'domain': self.get_domain(name),
                'color': color
            },
            'cancel': request.link(self.for_prev_step())
        }

    @property
    def config(self):
        return self.app.onboarding['onegov.town']

    def get_subdomain(self, name):
        return utils.normalize_for_url(name)

    def get_domain(self, name):
        return '{}.{}'.format(self.get_subdomain(name), self.config['domain'])

    def get_schema(self, name):
        return '{}-{}'.format(
            self.config['namespace'],
            self.get_subdomain(name).replace('-', '_')
        )

    def add_town(self, name, user, color, request):
        current_schema = self.app.session_manager.current_schema
        password = random_password(16)

        try:
            schema = self.get_schema(name)
            custom_config = self.config['configuration']

            self.app.session_manager.set_current_schema(schema)
            session = self.app.session_manager.session()

            if session.query(Organisation).first():
                raise AlreadyExistsError

            with self.app.temporary_depot(schema, **custom_config):
                create_new_organisation(self.app, name=name, reply_to=user)

            org = session.query(Organisation).first()
            org.theme_options['primary-color'] = color

            users = UserCollection(self.app.session_manager.session())
            assert not users.query().first()

            users.add(user, password, 'admin')

            title = request.translate(_("Welcome to OneGov Cloud"))
            welcome_mail = render_template('mail_welcome.pt', request, {
                'url': 'https://{}'.format(self.get_domain(name)),
                'mail': user,
                'layout': MailLayout(self, request),
                'title': title,
                'org': name
            })

            self.app.send_email(
                subject=title,
                receivers=(user, ),
                content=welcome_mail,
                reply_to='onegov@seantis.ch'
            )

        finally:
            self.app.session_manager.set_current_schema(current_schema)

        return {
            'info': [
                (_("Username"), user),
                (_("Password"), password),
            ],
            'url': 'https://{}'.format(self.get_domain(name))
        }



from onegov.onboarding import OnboardingApp
from onegov.onboarding.models import DefaultAssistant, TownAssistant


@OnboardingApp.path(model=DefaultAssistant, path='/')
def get_default_assistant(app):
    return DefaultAssistant(TownAssistant(app))


@OnboardingApp.path(model=TownAssistant,
                    path='/for-towns/{current_step_number}')
def get_town_assistant(app, current_step_number=1):
    try:
        return TownAssistant(app, current_step_number)
    except KeyError:
        return None



from onegov.form import Form
from onegov.onboarding import _
from wtforms import StringField, validators
from wtforms.fields.html5 import EmailField
from wtforms_components import ColorField


class TownForm(Form):
    """ First step when starting a new town. """

    # the name is limited to 63 characters because it has to fit into a
    # subdomain which may not exceed that length
    name = StringField(
        label=_("Town Name"),
        description=_("The name of your town (real or fictitious)"),
        validators=[validators.InputRequired(), validators.Length(max=63)],
        render_kw={
            'autofocus': '',
            'class_': 'autocomplete',
            'data-source': 'town-names'
        }
    )

    user = EmailField(
        label=_("E-Mail"),
        description=_("Your e-mail address"),
        validators=[validators.InputRequired(), validators.Email()]
    )

    color = ColorField(
        label=_("Primary Color"),
        validators=[validators.InputRequired()],
        default='#005ba1'
    )


class FinishForm(Form):
    pass



from hipchat import HipChat
from onegov.core import Framework, utils
from onegov.file import DepotApp
from onegov.libres import LibresIntegration
from onegov.onboarding import log
from onegov.onboarding.theme import OnboardingTheme


class OnboardingApp(Framework, LibresIntegration, DepotApp):

    serve_static_files = True

    def configure_onboarding(self, **cfg):
        self.onboarding = cfg['onboarding']
        assert 'onegov.town' in self.onboarding
        assert 'namespace' in self.onboarding['onegov.town']

    def notify_hipchat(self, message):
        if 'hipchat' in self.onboarding:
            try:
                hipchat = HipChat(token=self.onboarding['hipchat']['token'])
                hipchat.message_room(
                    room_id=self.onboarding['hipchat']['room_id'],
                    message_from='Onboarding',
                    message=message,
                    message_format='html',
                    color='green',
                    notify=True
                )
            except Exception:
                log.exception("Error during Hipchat message")


@OnboardingApp.static_directory()
def get_static_directory():
    return 'static'


@OnboardingApp.template_directory()
def get_template_directory():
    return 'templates'


@OnboardingApp.setting(section='core', name='theme')
def get_theme():
    return OnboardingTheme()


@OnboardingApp.setting(section='i18n', name='localedirs')
def get_i18n_localedirs():
    return [
        utils.module_path('onegov.onboarding', 'locale'),
        utils.module_path('onegov.form', 'locale'),
        utils.module_path('onegov.user', 'locale')
    ]


@OnboardingApp.setting(section='i18n', name='default_locale')
def get_i18n_default_locale():
    return 'de_CH'


@OnboardingApp.webasset_path()
def get_js_path():
    return 'assets/js'


@OnboardingApp.webasset_output()
def get_webasset_output():
    return 'assets/bundles'


@OnboardingApp.webasset('common')
def get_common_asset():
    yield 'modernizr.js'
    yield 'jquery.js'
    yield 'placeholder.js'
    yield 'fastclick.js'
    yield 'foundation.js'
    yield 'underscore.js'
    yield 'colorpicker.js'
    yield 'awesomeplete.js'
    yield 'common.js'
